// (C) 2001-2011 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Altera Program License Subscription 
// Agreement, Altera MegaCore Function License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Altera and sold by 
// Altera or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//
// Multiple PLL block, with a single output clock
// 
// Clock switching must use the exported reconfiguration ports
//
// $Header$
//

`timescale 1 ns / 1 ns

import altera_xcvr_functions::*;

module sv_xcvr_plls #(
  parameter plls                                  = 1,  // number of PLLs
  parameter refclks                               = 1,  // number of refclks per PLL
  parameter duty_cycle                            = 50,
  parameter output_clock_frequency                = "0 ps",
  parameter phase_shift                           = "0 ps",
  parameter reference_clock_frequency             = "0 ps",
  parameter sim_additional_refclk_cycles_to_lock  = 0,
  parameter enable_avmm                           = 0   // 1=include AVMM blocks

) (
  input   wire  [refclks -1:0]  refclk,
  input   wire  [plls    -1:0]  rst,
  input   wire  [plls    -1:0]  fbclk,
  
  output  wire  [plls    -1:0]  outclk,
  output  wire  [plls    -1:0]  locked,
  output  wire  [plls    -1:0]  fboutclk,

  // 
  input   wire  [plls*W_S5_RECONFIG_BUNDLE_TO_XCVR  -1 :0] reconfig_to_xcvr,
  output  wire  [plls*W_S5_RECONFIG_BUNDLE_FROM_XCVR-1 :0] reconfig_from_xcvr
);

localparam  w_bundle_to_xcvr  = W_S5_RECONFIG_BUNDLE_TO_XCVR;
localparam  w_bundle_from_xcvr= W_S5_RECONFIG_BUNDLE_FROM_XCVR;

genvar ig;      // Iterator for generated loops
generate
  for(ig=0; ig<plls; ig = ig + 1) begin: pll
    if(enable_avmm == 1) begin
      wire  [w_bundle_to_xcvr-1:0]  l_reconfig_to_xcvr;
      wire  [w_bundle_from_xcvr-1:0]l_reconfig_from_xcvr;

      // avalon MM native reconfiguration interfaces
      wire          avmm_reconfig_reset;     // one for each lane
      wire          avmm_reconfig_clk;       // one for each lane
      wire  [15:0]  avmm_reconfig_writedata; // Avalon DPRIO writedata
      wire  [10:0]  avmm_reconfig_address;   // Avalon DPRIO address
      wire  [ 1:0]  avmm_reconfig_byteen;    // two for each lane
      wire          avmm_reconfig_write;     // Avalon DPRIO write
      wire          avmm_reconfig_read;      // Avalon DPRIO read
      wire  [15:0]  avmm_reconfig_readdata;  // Avalon DPRIO readdata
      
      wire  [11:0]  pmatestbussel;
      wire  [23:0]  pmatestbus;
      
      wire          interface_sel;
      wire          ser_shift_load;
                       
      // Local generic mux signals
      wire  [5:0]   mux_sel;
      wire          mux_refclk;
      // Local avmm signals
      wire          mux_blockselect;
      wire  [15:0]  mux_readdata;
      wire          pll_blockselect;
      wire  [15:0]  pll_readdata;
      wire          local_avmm_rstn;
      wire          local_avmm_clk;
      wire  [15:0]  local_avmm_writedata;
      wire  [10:0]  local_avmm_address;
      wire  [ 1:0]  local_avmm_byteen;
      wire          local_avmm_write;
      wire          local_avmm_read;
      // Signals need for PLL AVMM adapter to PLL
      wire  [63:0]  writerefclkdata;
      wire  [63:0]  writeoutclkdata;
      wire  [63:0]  writephaseshiftdata;
      wire  [63:0]  writedutycycledata;
      wire  [63:0]  readrefclkdata;
      wire  [63:0]  readoutclkdata;
      wire  [63:0]  readphaseshiftdata;
      wire  [63:0]  readdutycycledata;

      assign  l_reconfig_to_xcvr  = reconfig_to_xcvr  [ig*w_bundle_to_xcvr  +:w_bundle_to_xcvr ];
      assign  l_reconfig_from_xcvr= reconfig_from_xcvr[ig*w_bundle_from_xcvr+:w_bundle_from_xcvr];

      // Extract and compose bundled signals
      sv_reconfig_bundle_to_xcvr #(
        .native_ifs         (1                  )   // number of native reconfig interfaces
      ) sv_reconfig_bundle_to_xcvr_inst(
        // bundled reconfig buses
        .reconfig_to_xcvr         (l_reconfig_to_xcvr     ),  // all inputs from reconfig block to native xcvr reconfig ports
        .reconfig_from_xcvr       (l_reconfig_from_xcvr   ),  // all outputs from native xcvr reconfig ports to reconfig block
        // native reconfig sources
        .native_reconfig_clk      (avmm_reconfig_clk      ),
        .native_reconfig_reset    (avmm_reconfig_reset    ),
        .native_reconfig_writedata(avmm_reconfig_writedata),
        .native_reconfig_address  (avmm_reconfig_address  ),
        .native_reconfig_write    (avmm_reconfig_write    ),
        .native_reconfig_read     (avmm_reconfig_read     ),
        // native reconfig sinks
        .native_reconfig_readdata (avmm_reconfig_readdata ),  // Avalon DPRIO readdata

        .pif_testbus              (pmatestbus             ),
        .pif_testbus_adce_done    (/*unused*/             ),
        .pif_hardoccaldone        (/*unused*/             ),
        .pif_eyemonitor           (/*unused*/             ),
        .pif_testbus_sel          (pmatestbussel          ),
        .pif_testbus_adce_capture (/*unused*/             ),
        .pif_testbus_adce_standby (/*unused*/             ),
        .pif_hardoccalen          (/*unused*/             ),

        .pif_interface_sel        (interface_sel          ),
        .pif_ser_shift_load       (ser_shift_load         )
      );

      // Instantiate AVMM_INTERFACE
      stratixv_hssi_avmm_interface stratixv_hssi_avmm_interface_inst (
        .avmmrstn           (~avmm_reconfig_reset     ),
        .avmmclk            (avmm_reconfig_clk        ),
        .avmmwrite          (avmm_reconfig_write      ),
        .avmmread           (avmm_reconfig_read       ),
        .avmmbyteen         (avmm_reconfig_byteen     ),
        .avmmaddress        (avmm_reconfig_address    ),
        .avmmwritedata      (avmm_reconfig_writedata  ),
        .avmmreaddata       (avmm_reconfig_readdata   ),

        .blockselect        ({{88{1'b0}} ,mux_blockselect ,pll_blockselect} ),
        .readdatachnl       ({{88{16'd0}},mux_readdata    ,pll_readdata   } ),
        .clkchnl            (local_avmm_clk           ),
        .rstnchnl           (local_avmm_rstn          ),
        .writedatachnl      (local_avmm_writedata     ),
        .regaddrchnl        (local_avmm_address       ),
        .writechnl          (local_avmm_write         ),
        .readchnl           (local_avmm_read          ),
        .byteenchnl         (local_avmm_byteen        ),
  
        // The following ports belong to pm_tst_mux blocks in the PMA
        .pmatestbus         (pmatestbus               ),
        .pmatestbussel      (pmatestbussel            ),

        //
        .scanmoden          (1'b1                     ),
        .scanshiftn         (1'b1                     ),
        .sershiftload       (ser_shift_load           ),
        .interfacesel       (interface_sel            ),

        .refclkdig          (/*unused*/ ),
        .avmmreserevdin     (/*unused*/ ),

        .avmmreservedout    (/*unused*/ ),
        .dpriorstntop       (/*unused*/ ),
        .dprioclktop        (/*unused*/ ),
        .mdiodistopchnl     (/*unused*/ ),
        .dpriorstnmid       (/*unused*/ ),
        .dprioclkmid        (/*unused*/ ),
        .mdiodismidchnl     (/*unused*/ ),
        .dpriorstnbot       (/*unused*/ ),
        .dprioclkbot        (/*unused*/ ),
        .mdiodisbotchnl     (/*unused*/ ),
        .dpriotestsitopchnl (/*unused*/ ),
        .dpriotestsimidchnl (/*unused*/ ),
        .dpriotestsibotchnl (/*unused*/ )

      ); 
      
      // Instantantiate Generic MUX Avalon Adapter
      generic_mux_avalon_adapter generic_mux_avalon_adapter_inst (
        .reset              (~local_avmm_rstn     ),
        .clk                (local_avmm_clk       ),
        .write              (local_avmm_write     ),
        .read               (local_avmm_read      ),
        .byteenable         (local_avmm_byteen    ),
        .address            (local_avmm_address   ),
        .writedata          (local_avmm_writedata ),
        .blockselectinternal(mux_blockselect      ),
        .readdata           (mux_readdata         ),
        .selout             (mux_sel              )
      );

      // Instantiate Generic PLL Avalon Adapter
      generic_pll_avalon_adapter generic_pll_avalon_adapter_inst (
        .reset              (~local_avmm_rstn     ),
        .clk                (local_avmm_clk       ),
        .write              (local_avmm_write     ),
        .read               (local_avmm_read      ),
        .byteenable         (local_avmm_byteen    ),
        .address            (local_avmm_address   ),
        .writedata          (local_avmm_writedata ),
        .blockselectinternal(pll_blockselect      ),
        .readdata           (pll_readdata         ),

        .writerefclkdata    (readrefclkdata       ),
        .writeoutclkdata    (readoutclkdata       ),
        .writephaseshiftdata(readphaseshiftdata   ),
        .writedutycycledata (readdutycycledata    ),
        .readrefclkdata     (writerefclkdata      ),
        .readoutclkdata     (writeoutclkdata      ),
        .readphaseshiftdata (writephaseshiftdata  ),
        .readdutycycledata  (writedutycycledata )
      );
        

      // Instantiate the Generic MUX
      generic_mux generic_mux_inst (
        .din  (refclk           ),  // Each PLL gets all refclks
        .sel  (mux_sel          ),
        .dout (mux_refclk       )   // Selected refclk
      );

      // Create generic TX pll
      generic_pll #(  
        .reference_clock_frequency(reference_clock_frequency),
        .output_clock_frequency   (output_clock_frequency   )
      ) tx_pll (
        .fbclk              (fbclk    [ig]      ),
        .fboutclk           (fboutclk [ig]      ),
        .refclk             (mux_refclk         ),
        .rst                (rst      [ig]      ),
        .outclk             (outclk   [ig]      ),
        .locked             (locked   [ig]      ),

        // Inputs from Generic PLL Adapter
        .writerefclkdata    (writerefclkdata    ),
        .writeoutclkdata    (writeoutclkdata    ),
        .writephaseshiftdata(writephaseshiftdata),
        .writedutycycledata (writedutycycledata ),
        // Outputs to Generic PLL Adapter 
        .readrefclkdata     (readrefclkdata     ),
        .readoutclkdata     (readoutclkdata     ),
        .readphaseshiftdata (readphaseshiftdata ),
        .readdutycycledata  (readdutycycledata  )
      );
    end else begin
      assign  reconfig_from_xcvr= {(plls*w_bundle_from_xcvr){1'b0}};
      // Create generic TX pll
      generic_pll #(  
        .reference_clock_frequency(reference_clock_frequency),
        .output_clock_frequency   (output_clock_frequency   )
      ) tx_pll (
        .fbclk              (fbclk    [ig]  ),
        .fboutclk           (fboutclk [ig]  ),
        .refclk             (refclk   [ 0]  ),
        .rst                (rst      [ig]  ),
        .outclk             (outclk   [ig]  ),
        .locked             (locked   [ig]  ),

        // Inputs from Generic PLL Adapter
        .writerefclkdata    (/*unused*/     ),
        .writeoutclkdata    (/*unused*/     ),
        .writephaseshiftdata(/*unused*/     ),
        .writedutycycledata (/*unused*/     ),
        // Outputs to Generic PLL Adapter
        .readrefclkdata     (/*unused*/     ),
        .readoutclkdata     (/*unused*/     ),
        .readphaseshiftdata (/*unused*/     ),
        .readdutycycledata  (/*unused*/     )
      );
    end
  end
endgenerate

endmodule
